package com.example.gertec_tsg800;

import android.Manifest;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.Build;
import android.util.Base64;

import androidx.annotation.NonNull;
import androidx.annotation.RequiresApi;

import com.google.zxing.integration.android.IntentIntegrator;
import com.google.zxing.integration.android.IntentResult;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;

import br.com.gertec.gedi.exceptions.GediException;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry;
import io.flutter.plugin.common.PluginRegistry.Registrar;

import static android.app.Activity.RESULT_OK;
import static android.hardware.Camera.Parameters.FLASH_MODE_ON;

/**
 * GertecPlugin
 */
public class GertecTsg800Plugin implements
        FlutterPlugin,
        PluginRegistry.ActivityResultListener,
        PluginRegistry.RequestPermissionsResultListener,
        ActivityAware,
        MethodCallHandler
    {
    public static final String G700 = "GPOS700";
    public static final String G800 = "Smart G800";
    private static final String version = "RC03";
    public static String Model = Build.MODEL;
    private Context applicationContext;
    private MethodChannel methodChannel;
    private IntentIntegrator qrScan;
    private static Result pendingResult;
    private ConfigPrint configPrint;
    private GertecPrinter gertecPrinter;
    private Activity myActivity;
    private String resultado_Leitor;
    public static ActivityPluginBinding _binding;

    private String tipo;
    private static final int REQUEST_CODE_SCAN_ACTIVITY = 2777;
    private static final int REQUEST_CODE_CAMERA_PERMISSION = 3777;
    private ArrayList<String> arrayListTipo;

    /**
     * Registrar plugin em projetos que não tenham a incorporação V2 Android
     **/

    public GertecTsg800Plugin() {}
    public static void registerWith(Registrar registrar) {
        final GertecTsg800Plugin instance = new GertecTsg800Plugin();
        instance.onAttachedToEngine(registrar.activeContext(), registrar.messenger());
    }

    @RequiresApi(api = Build.VERSION_CODES.KITKAT)
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        if (_binding != null) {
            _binding.addActivityResultListener(this);
        }
        Intent intent = null;
        String mensagem = "";
        pendingResult = null;
        pendingResult = result;
        try {
            switch (call.method) {
                case "lerNfc":
                    try {
                        intent = new Intent(myActivity, NfcLeitura.class);
                        myActivity.startActivityForResult(intent, 111);
                    } catch (Exception e) {
                        e.printStackTrace();
                        result.notImplemented();
                    }
                    break;
                case "gravarNfc":
                    String mensagemGravar = call.argument("mensagemGravar");
                    try {
                        intent = new Intent(myActivity, NfcGravacao.class);
                        intent.putExtra("mensagemGravar", mensagemGravar);
                        myActivity.startActivity(intent);
                        result.success("Gravou mensagem");
                    } catch (Exception e) {
                        e.printStackTrace();
                        result.notImplemented();
                    }
                    break;
                case "formatarNfc":
                    try {
                        intent = new Intent(myActivity, NfcFormatar.class);
                        myActivity.startActivity(intent);
                        result.success("Formatou cartão");
                    } catch (Exception e) {
                        e.printStackTrace();
                        result.notImplemented();
                    }
                    break;
                case "testeNfc":
                    try {
                        intent = new Intent(myActivity, NfcLerGravar.class);
                        myActivity.startActivity(intent);
                        result.success("Testou cartão");
                    } catch (Exception e) {
                        e.printStackTrace();
                        result.notImplemented();
                    }
                    break;
                case "statusImpressora":
                    try {
                        gertecPrinter.getStatusImpressora();
                        result.success(gertecPrinter.isImpressoraOK());
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                    break;
                case "fimimpressao":
                    try {
                        gertecPrinter.ImpressoraOutput();
                        result.success("Finalizou impressao");
                    } catch (GediException e) {
                        e.printStackTrace();
                    }
                    break;
                case "avancarLinha":
                    try {
                        this.gertecPrinter.avancaLinha(call.argument("quantLinhas"));
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                    break;
                case "imprimirTexto":
                    gertecPrinter.getStatusImpressora();
                    if (gertecPrinter.isImpressoraOK()) {
                        mensagem = call.argument("mensagem");
                        String alinhar = call.argument("alinhar");
                        int size = call.argument("size");
                        String fontFamily = call.argument("font");
                        Boolean negrito = call.argument("negrito");
                        Boolean italico = call.argument("italico");
                        Boolean sublinhado = call.argument("sublinhado");

                        if(fontFamily.equals("MONOSPACE") ||fontFamily.equals("SANS SERIF") || fontFamily.equals("SERIF")){
                            this.configPrint.setFonte(fontFamily);
                        }else{
                            if(call.argument("ext").equals(".otf")){
                                this.configPrint.setFonte(CreatenewFonteImpressao(fontFamily, "otf"));
                            }
                            else if(call.argument("ext").equals(".ttf")){
                                this.configPrint.setFonte(CreatenewFonteImpressao(fontFamily, "ttf"));
                            }
                        }
                        this.configPrint.setTamanho(size);
                        this.configPrint.setAlinhamento(alinhar);
                        this.configPrint.setNegrito (negrito);
                        this.configPrint.setItalico(italico);
                        this.configPrint.setSublinhado(sublinhado);
                        this.gertecPrinter.setConfigImpressao(configPrint);
                        this.gertecPrinter.imprimeTexto(mensagem);
                    }
                    result.success("Imprimiu Texto");
                    break;
                case "imprimirImagem":
                    gertecPrinter.getStatusImpressora();
                    if (gertecPrinter.isImpressoraOK()) {
                        if(call.argument("width") != null && call.argument("height") != null){
                            configPrint.setiWidth(call.argument("width"));
                            configPrint.setiHeight(call.argument("height"));
                        }
                        gertecPrinter.setConfigImpressao(configPrint);
                        gertecPrinter.imprimeImagem(call.argument("codigoImagem"));
                    }
                    result.success("Imprimiu Imagem");
                    break;
                case "leitorV1":
                    tipo = call.argument("tipoLeitura");
                    startCamera();
                    break;
                case "leitorV2":
                    intent = new Intent(myActivity, CodigoBarrasV2.class);
                    myActivity.startActivity(intent);
                    result.success("Camera Aberta");
                    break;
                case "imprimirCodigoDeBarra":
                    gertecPrinter.getStatusImpressora();
                    if (gertecPrinter.isImpressoraOK()) {
                        mensagem = call.argument("mensagem");
                        int height = call.argument("height");
                        int width = call.argument("width");
                        String barCode = call.argument("barCode");
                        gertecPrinter.imprimeBarCode(mensagem, height, width, barCode);
                    }
                    result.success("Imprimiu Código de Barra");
                    break;
            }
        } catch (Exception e) {
            e.getLocalizedMessage();
        }
    }

    /**
     * Registrar plugin em projetos que não tenham a incorporação V2 Android
     **/
    public void onAttachedToEngine(FlutterPluginBinding binding) {
        onAttachedToEngine(binding.getApplicationContext(), binding.getBinaryMessenger());
        this.arrayListTipo = new ArrayList<String>();
    }

    private void onAttachedToEngine(Context applicationContext, BinaryMessenger messenger) {
        this.applicationContext = applicationContext;
        methodChannel = new MethodChannel(messenger, "gertec_tsg800");
        methodChannel.setMethodCallHandler(this);
    }

    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    }

    @RequiresApi(api = Build.VERSION_CODES.KITKAT)
    private String CreatenewFonteImpressao(String base64Fonte, String ext) throws IOException {
        // Cria um arquivo XML
        File file = new File( myActivity.getFilesDir().getPath() + "/novaFonte."+ext);

        FileOutputStream fos = new FileOutputStream(file);
        fos.write(Base64.decode(base64Fonte, Base64.NO_WRAP));
        fos.close();

        return file.getPath();
    }
    private void startCamera() {
        this.arrayListTipo.add(this.tipo);
        qrScan = new IntentIntegrator(this.myActivity);
        qrScan.setPrompt("Digitalizar o código ");
        qrScan.setBeepEnabled(true);
        qrScan.setBarcodeImageEnabled(true);
        qrScan.setTimeout(30000); // 30 * 1000 => 3 minuto
        qrScan.addExtra("FLASH_MODE_ON", FLASH_MODE_ON);
        qrScan.initiateScan(this.arrayListTipo);
    }

    /**
     * Registrar Activity
     **/
    public void onAttachedToActivity(ActivityPluginBinding binding) {
        _binding = binding;
        this.myActivity = binding.getActivity();
        gertecPrinter = new GertecPrinter(myActivity);
        configPrint = new ConfigPrint(myActivity);
        gertecPrinter.setConfigImpressao(configPrint);
    }

    public void onDetachedFromActivityForConfigChanges() {
    }

    public void onReattachedToActivityForConfigChanges(ActivityPluginBinding binding) {
    }

    public void onDetachedFromActivity() {
    }

    @RequiresApi(api = Build.VERSION_CODES.M)
    public boolean onRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) {
        myActivity.requestPermissions(new String[]{Manifest.permission.CAMERA}, REQUEST_CODE_CAMERA_PERMISSION);
        return false;
    }

    public boolean onActivityResult(int requestCode, int resultCode, Intent data) {
        if (requestCode == 111) {
            if (resultCode == RESULT_OK && data != null) {
                pendingResult.success(data.getStringExtra("mensagemLeitura"));
            } else {
                pendingResult.notImplemented();
            }
        } else {
            IntentResult intentResult = IntentIntegrator.parseActivityResult(requestCode, resultCode, data);
            if (intentResult != null) {
                // if qrcode has nothing in it
                if (intentResult.getContents() == null) {
                    // Toast.makeText(this, "Result Not Found", Toast.LENGTH_LONG).show();
                    resultado_Leitor = (this.tipo + ": Não foi possível ler o código.\n");
                } else {
                    // if qr contains data
                    try {
                        resultado_Leitor = this.tipo + ": " + intentResult.getContents() + "\n";
                    } catch (Exception e) {
                        e.printStackTrace();
                        resultado_Leitor = this.tipo + ": Erro " + e.getMessage() + "\n";
                    }
                }
            } else {
                resultado_Leitor = this.tipo + ": Não foi possível fazer a leitura.\n";
            }
            pendingResult.success(resultado_Leitor);
            this.arrayListTipo.clear();
        }
        return false;
    }
}